#include <stdio.h>
#include <unistd.h>
#include <math.h>
#include "idc.h"

/**********************************************************************
*                                                                      *
*  File: histimg.c                                                     *
*  Function:  find the histogram of an image or portions of an image   *
*  Author (aka person to blame) : K. Sayood                            *
*  Last mod: 7/15/95                                                   *
*  Usage:  see usage(), for details see histimg.doc or man  page    *
***********************************************************************/

void usage();

void main(int argc, char **argv)
{
    int row, col, row_size, col_size, hist[8][512], howmuch, histjump,j;
    char input_image[50], qimage[50];
    unsigned char **imgin, **block;
    int *bound, input, label, numhist, numbits, c, i, rowblocks;
    int colblocks, brow, bcol, block_count;
    int minval, maxval, range, level, stepsize, temp, end_flag, blocksize;
    FILE *ofp;
  extern int  optind;
  extern char *optarg;




    fprintf(stderr,"\n\n\t\tForward Adaptive Quantization of Images - Encoder\n");

  ofp = stdout;
  strcpy(qimage,"standard out");
  blocksize = 4;
  row_size = -1;
  col_size = -1;
  howmuch = 0;

  while((c=getopt(argc,argv,"i:o:l:b:x:y:h"))!=EOF)
  {
   switch (c){
   case 'i':
         strcpy(input_image,optarg);
         break;
   case 'o':
         strcpy(qimage,optarg);
         ofp = fopen(optarg,"w");
         break;
   case 'l':
         sscanf(optarg,"%d", &howmuch);
         break;
   case 'b':
         sscanf(optarg,"%d",&blocksize);
         break;
   case 'x':
         sscanf(optarg,"%d",&row_size);
         break;
   case 'y':
         sscanf(optarg,"%d",&col_size);
         break;
   case 'h':
         usage();
         exit(1);
              }
   }



     

    


    if(row_size<0 || col_size<0)
      image_size(input_image, &row_size, &col_size);   

/* Allocate space for input image  */

    imgin  = (unsigned char **) calloc(row_size,sizeof(char *));  
    for(row=0; row<row_size; row++)
     imgin[row] = (unsigned char *) calloc((col_size),sizeof(char));


/* Read the input image */

    readimage(input_image, imgin, row_size, col_size);  

    histjump = col_size / (howmuch+1);

/*  Main encoding loop  */

   for(numhist=0; numhist<= howmuch; numhist++)
   {
   for(row=numhist*histjump; row<numhist*histjump+blocksize; row++)
      for(col=1; col<col_size; col++)
         {
          i = imgin[row][col] - imgin[row][col-1] + 255;
          hist[numhist][i] += 1;
          }
    }

    for(i=0; i<512; i++)
     {
      fprintf(ofp,"  %d  ",i-255);
      for(j=0; j<howmuch+1; j++)
       fprintf(ofp,"  %d   ",hist[j][i]);
      fprintf(ofp,"\n");
     }

}
void usage()
{
  fprintf(stderr,"Usage: aqfimg_enc [-i input file][-o output file] [-l numlev] [-b numbits] [-s blocksize]\n");
  fprintf(stderr,"\t input file : File containing the image to be quantized.\n");
  fprintf(stderr,"\t output file: File which will contain the compressed representation\n");
  fprintf(stderr,"\t including any header information. The default is standard out.\n");
  fprintf(stderr,"\t numlev  : Number of reconstruction values in the quantizer. If\n");
  fprintf(stderr,"\t this option is not used, the number of levels is computed as\n");
  fprintf(stderr,"\t 2^(number of bits).  If numlev is provided, numbits need not\n");
  fprintf(stderr,"\t be provided.\n");
  fprintf(stderr,"\t numbits  : Number of bits per pixel.\n");
  fprintf(stderr,"\t row_size: Length of a row of the image\n");
  fprintf(stderr,"\t col_size: Length of a column of the image\n");
  fprintf(stderr,"\t\t If the column or row size is not provided the program\n")
;
  fprintf(stderr,"\t\t will check to see if the iamge corresponds to any of the\
n");
  fprintf(stderr,"\t\t standard sizes it is familiar with.  To add to the list o
f\n");
  fprintf(stderr,"\t\t standard sizes, edit image_size.c\n\n");





}
